package com.peaceray.codeword.data.source

import com.peaceray.codeword.data.model.version.SupportedVersions
import retrofit2.Response
import retrofit2.http.GET

/**
 * The API interface used by CodeWord.
 *
 * As of this writing, only one API call is ever used: "/versions". On the server end this
 * is just a text file.
 *
 * The Daily challenge is generated by the client app based on the daily number as an input seed.
 * For consistency across all app installations, it is important that changes to the Daily
 * generation algorithm are applied uniformly across the userbase (or two different people will
 * get different Dailies on the same day). When new logic for Daily generation is introduced, it
 * will be scheduled in the app code for a switchover on a specific future date; before that date
 * arrives (but a reasonable period after app release) the content available at the "/versions"
 * endpoint will be updated to indicate the new minimum app version. Users must update to this
 * version to access the Daily puzzle.
 */
interface CodeWordApi {

    @GET("versions")
    suspend fun getVersions(): Response<SupportedVersions>

}